import { LevelInfo, LevelsMap } from 'rfc-log-levels'
import { LineOffset, LineInfo } from 'get-current-line'
import { Transform, TransformConfig } from './transform.js'
/** The log entry that Caterpillar creates and forwards to its transforms */
export interface LogEntry extends LevelInfo, LineInfo {
	/** the iso string of when the log occured */
	date: string
	/** all the arguments that were after the log level */
	args: any[]
}
/** Configuration for the Caterpillar Logger */
export interface LoggerConfig extends TransformConfig {
	/**
	 * The configuration to use for the line offset.
	 * This defaults to the caterpillar filename, and any method that includes the word `log`. */
	lineOffset: LineOffset
	/**
	 * The configuration to use for the log levels.
	 * Defaults to the RFC Log Levels with the default log level set to the info log level `6`.
	 */
	levels: LevelsMap
	/**
	 * Only fetch line information for entries that have a log level equal to, or below this number.
	 * You should only specify this if you need it, as fFetching line information for thousands of log entries, which is typical in large applications, will slow your application down dramatically.
	 * If not specified, defaults to `-1` which effectively disables this functionality.
	 */
	lineLevel: number
}
/**
 * Logger.
 * This is what we write to.
 * @example Creation
 * ``` javascript
 * // Via class
 * import { Logger } from 'caterpillar'
 * const logger = new Logger()
 * ```
 */
export declare class Logger extends Transform {
	config: LoggerConfig
	constructor(opts?: Partial<LoggerConfig>)
	/**
	 * Takes an arguments array and tranforms it into a log entry
	 * @param args
	 */
	format(args: any): LogEntry
	/**
	 * Log the arguments into the logger stream as formatted data with debugging information.
	 * Such that our transformers can deal with it intelligently.
	 *
	 * @example Inputs
	 * ``` javascript
	 * logger.log('note', 'this is working swell')
	 * ```
	 * ``` javascript
	 * logger.log('this', 'worked', 'swell')
	 * ```
	 *
	 * @example Results
	 * ``` json
	 * {
	 * 	"args": ["this is working swell"],
	 * 	"date": "2013-04-25T10:18:25.722Z",
	 * 	"levelNumber": 5,
	 * 	"levelName": "notice",
	 * 	"line": "59",
	 * 	"method": "Object.<anonymous>",
	 * 	"file": "/Users/balupton/some-project/calling-file.js"
	 * }
	 * ```
	 * ``` json
	 * {
	 *		"args": ["this", "worked", "well"],
	 *		"date": "2013-04-25T10:18:26.539Z",
	 *		"levelNumber": 6,
	 *		"levelName": "info",
	 *		"line": "60",
	 *		"method": "Object.<anonymous>",
	 *		"file": "/Users/balupton/some-project/calling-file.js"
	 * }
	 * ```
	 */
	log(...args: any): void
	/** Alias for log which prefixes the error log level */
	error(...args: any): void
	/** Alias for log which prefixes the warn log level */
	warn(...args: any): void
	/** Alias for log which prefixes the info log level */
	info(...args: any): void
	/** Alias for log which prefixes the debug log level */
	debug(...args: any): void
}
export default Logger
//# sourceMappingURL=logger.d.ts.map
