import { LogEntry } from './logger.ts'
import { Transform, TransformConfig } from './transform.ts'

/** Configuration for the Caterpillar Filter Transform */
export interface FilterConfig extends TransformConfig {
	/**
	 * Only display entries that have a log level below or equal to this number.
	 * Defaults to the info log level `6`.
	 */
	filterLevel: number
}

/**
 * Caterpillar Filter Transform.
 * Filters the log entries, keeping only those equal to or below the specified `filterLevel`.
 * @example
 * ``` javascript
 * import { Logger, Filter } from 'caterpillar'
 * const logger = new Logger()
 * const filter = new Filter({ filterLevel: 6 })
 * logger.pipe(filter).pipe(process.stdout)
 * logger.log('info', 'this will be outputted')
 * logger.log('debug', 'this will be ignored')
 * filter.config.filterLevel = 5
 * logger.log('info', 'now even this will be ignored')
 * logger.log('note', 'but not this')
 * ```
 */
export class Filter extends Transform {
	config!: FilterConfig
	constructor(opts?: Partial<FilterConfig>) {
		super(opts)
		this.config.filterLevel = opts?.filterLevel ?? 6
	}

	/** Retain only log entries that are equal to or less than the specified filter level. */
	format(entry: LogEntry): LogEntry | null {
		return entry.levelNumber <= this.config.filterLevel ? entry : null
	}
}

export default Filter
