import getLogLevel, { rfcLogLevels } from 'rfc-log-levels'
import getCurrentLine from 'get-current-line'
import { Transform } from './transform.js'
/**
 * Logger.
 * This is what we write to.
 * @example Creation
 * ``` javascript
 * // Via class
 * import { Logger } from 'caterpillar'
 * const logger = new Logger()
 * ```
 */
export class Logger extends Transform {
	constructor(opts) {
		super(opts)
		this.config.lineOffset = opts?.lineOffset ?? {
			file: __filename,
			method: /log/i,
		}
		this.config.levels = Object.assign(
			{},
			rfcLogLevels,
			{
				default: 6,
			},
			opts?.levels
		)
		this.config.lineLevel = opts?.lineLevel ?? -1
	}
	/**
	 * Takes an arguments array and tranforms it into a log entry
	 * @param args
	 */
	format(args) {
		const { lineOffset, levels, lineLevel } = this.config
		const level = args.shift()
		let levelInfo = getLogLevel(level, levels)
		if (levelInfo == null) {
			levelInfo = getLogLevel('default', levels)
			if (levelInfo == null) {
				throw new Error(
					'caterpillar: rfc-log-levels: no default log level configuration was provided'
				)
			}
			args.unshift(level)
		}
		const date = new Date().toISOString()
		const lineInfo =
			levelInfo.levelNumber <= lineLevel
				? getCurrentLine(lineOffset)
				: {
						line: -1,
						method: 'unknown',
						file: 'unknown',
				  }
		return Object.assign({ date, args }, levelInfo, lineInfo)
	}
	/**
	 * Log the arguments into the logger stream as formatted data with debugging information.
	 * Such that our transformers can deal with it intelligently.
	 *
	 * @example Inputs
	 * ``` javascript
	 * logger.log('note', 'this is working swell')
	 * ```
	 * ``` javascript
	 * logger.log('this', 'worked', 'swell')
	 * ```
	 *
	 * @example Results
	 * ``` json
	 * {
	 * 	"args": ["this is working swell"],
	 * 	"date": "2013-04-25T10:18:25.722Z",
	 * 	"levelNumber": 5,
	 * 	"levelName": "notice",
	 * 	"line": "59",
	 * 	"method": "Object.<anonymous>",
	 * 	"file": "/Users/balupton/some-project/calling-file.js"
	 * }
	 * ```
	 * ``` json
	 * {
	 *		"args": ["this", "worked", "well"],
	 *		"date": "2013-04-25T10:18:26.539Z",
	 *		"levelNumber": 6,
	 *		"levelName": "info",
	 *		"line": "60",
	 *		"method": "Object.<anonymous>",
	 *		"file": "/Users/balupton/some-project/calling-file.js"
	 * }
	 * ```
	 */
	log(...args) {
		this.write(args)
	}
	/** Alias for log which prefixes the error log level */
	error(...args) {
		this.write(['error', ...args])
	}
	/** Alias for log which prefixes the warn log level */
	warn(...args) {
		this.write(['warn', ...args])
	}
	/** Alias for log which prefixes the info log level */
	info(...args) {
		this.write(['info', ...args])
	}
	/** Alias for log which prefixes the debug log level */
	debug(...args) {
		this.write(['debug', ...args])
	}
}
export default Logger
